@@ -68,7 +68,10 @@ import collections as _collections
 import datetime as _datetime
 import inspect as _inspect
 import os as _os
-import cPickle as _pickle
+try:
+    import cPickle as _pickle
+except ImportError:
+    import pickle as _pickle
 import shelve as _shelve
 import sys as _sys
 import time as _time
@@ -210,7 +213,7 @@ class FileBackend(object):
         return _os.path.isfile(self._get_filename(key))
 
     def __getitem__(self, key):
-        return _pickle.load(open(self._get_filename(key)))
+        return _pickle.load(open(self._get_filename(key), 'rb'))
 
     def __setitem__(self, key, value):
         # first-write wins semantics.  if someone else already cached this
@@ -220,7 +223,7 @@ class FileBackend(object):
         import portalocker
 
         try:
-            file = open(self._get_filename(key), 'w')
+            file = open(self._get_filename(key), 'wb')
             portalocker.Lock(file)
             try:
                 _pickle.dump(value, file, _pickle.HIGHEST_PROTOCOL)
@@ -242,7 +245,7 @@ class FileBackend(object):
 
     def _get_filename(self, key):
         # hash the key and use as a filename
-        return self.dir_name + '/' + hashlib.sha512(key).hexdigest()
+        return self.dir_name + '/' + hashlib.sha512(key.encode()).hexdigest()
 
 
 class DictBackend(dict):
@@ -363,6 +366,17 @@ def function_with_cache(function, *args, **kwargs):
     return retval
 
 
+def is_class(x):
+    """ handle difference between py2 and py3 """
+    if isinstance(x, type):
+        return True
+
+    if hasattr(_types, 'ClassType'):
+        return isinstance(x, _types.ClassType)
+    else:
+        return False
+
+
 def functioncache(
         seconds_of_validity=None,
         fail_silently=True,
@@ -375,7 +389,7 @@ def functioncache(
     function_key allows you to introduce cache invalidation on implementation changes
     '''
     # if a class is passed in, create an instance of that class as the backend
-    if isinstance(backend, _types.ClassType) or isinstance(backend, type):
+    if is_class(backend):
         backend = backend()
 
     def functioncache_decorator(function):
@@ -425,8 +439,6 @@ def memcachecache(seconds_of_validity=None, fail_silently=False, mc=None):
     )
 
 if _os.path.exists(_os.path.expanduser("~/.disable_functioncache")):
-    print 'disabled functioncache'
-
     def functioncache(*_, **__):
         def nop_decorator(function):
             return function
